from urllib.parse import quote
import re
from .query_parser import QueryParser
from .config import ResourceConfig
from .diagnostics import Diagnostic, DiagnosticTypes

class TsakorpusQueryParser(QueryParser):
    """
    Parses search queries for Tsakorpus-based corpora.
    """

    rxTsakorpusBool = re.compile('[()|,]')

    def build_get_string(self, getParams, config):
        """
        Build a GET string (everything after the ?) from a description
        of the GET parameters in the getParams list.
        """
        nWords = len(self.term_indexes(getParams))
        s = 'n_words=' + str(nWords)
        for param in getParams:
            if type(param[1]) is list:
                index = '_'.join(str(v) for v in param[1])
            else:
                index = str(param[1])
            s += '&' + param[0] + index + '=' + quote(param[2])
        s += '&page_size=' + str(config.max_hits)
        return s

    def term_query(self, query, config):
        """
        Return list of query parameters for one term or sequence of terms.
        """
        if len(query) >= 2 and query.startswith('"') and query.endswith('"'):
            query = query[1:len(query)-1]
        if len(query) <= 0:
            return Diagnostic(DiagnosticTypes.sru, 10)
        getParams = []
        iTerm = 0
        for term in query.split(' '):
            if len(term) > 0:
                iTerm += 1
                getParams.append(['wf', iTerm, term])
                if iTerm >= 2:
                    getParams.append(['word_rel_', [iTerm-1, iTerm], '1'])
                    getParams.append(['word_dist_from_', [iTerm-1, iTerm], '1'])
                    getParams.append(['word_dist_to_', [iTerm-1, iTerm], '1'])
        return getParams


    def term_indexes(self, getParams):
        """
        Find all search term indexes used in the GET parameters
        specified by getParams list. Return list of integers (1-based).
        """
        terms = set()
        for param in getParams:
            if type(param[1]) is int:
                terms.add(param[1])
            elif type(param[1]) is list:
                for t in param[1]:
                    terms.add(t)
        return [t for t in sorted(terms)]


    def shift_term_indexes(self, getParams, shift):
        """
        Increase all search term indexes in the GET parameters
        specified by getParams by shift.
        """
        getParamsShifted = []
        for param in getParams:
            if type(param[1]) is int:
                newParam = (param[0], param[1] + shift, param[2])
            else:
                newParam = (param[0], [i + shift for i in param[1]], param[2])
            getParamsShifted.append(newParam)
        return getParamsShifted


    def binary_bool(self, strOp, operandL, operandR, config):
        if len(operandL) <= 0 or len(operandR) <= 0:
            raise Diagnostic(DiagnosticTypes.sru, 10)
        termsL = self.term_indexes(operandL)
        operandR = self.shift_term_indexes(operandR, max(termsL))
        termsR = self.term_indexes(operandR)
        if strOp == 'AND':
            if len(termsL) > 1 and len(termsR) > 1:
                message = 'Tsakorpus does not support queries that combine several ' \
                          'multi-word sequences with boolean operators.'
                raise Diagnostic(DiagnosticTypes.sru, 48, message=message)
            return operandL + operandR
        elif strOp == 'OR':
            if len(termsL) > 1 or len(termsR) > 1:
                message = 'Tsakorpus does not support queries that combine several ' \
                          'multi-word sequences with boolean operators.'
                raise Diagnostic(DiagnosticTypes.sru, 48, message=message)
            if operandL[0][0] != 'wf' or operandR[0][0] != 'wf':
                raise Diagnostic(DiagnosticTypes.sru, 47)
            if self.rxTsakorpusBool.search(operandL[0][2]) is not None:
                getParamsNew = [('wf', operandL[0][1], '(' + operandL[0][2] + ')|' + operandR[0][2])]
            else:
                getParamsNew = [('wf', operandL[0][1], operandL[0][2] + '|' + operandR[0][2])]
            return getParamsNew
        raise Diagnostic(DiagnosticTypes.sru, 37, details=strOp)

    def translate_fcsql(self, query: str, config: ResourceConfig, basicSearch: bool = False, start=0, end=-1):
        """
        Translate an FCS-QL query into a Tsakorpus GET query.
        If something is wrong with the query, raise a Diagnostic exception.
        The function is recursive and only looks at the part of the string
        delimited by start and end parameters.
        """
        if end == -1:
            # Top-level call, so return a finalized GET string
            end = len(query)
            if end == 0:
                raise Diagnostic(DiagnosticTypes.sru, 27)
            if self.rxTermQuery.search(query) is not None:
                return self.build_get_string(self.term_query(query, config), config)
            return self.build_get_string(self.translate_fcsql(query, config,
                                                              basicSearch=basicSearch,
                                                              start=start, end=end),
                                         config)
            # if query.count('(') != query.count(')'):
            #     return None
        if len(query) <= 0:
            raise Diagnostic(DiagnosticTypes.sru, 27)
        if start >= len(query) - 1 or end <= 0:
            raise Diagnostic(DiagnosticTypes.sru, 10)
        while start < len(query) and query[start] in ' \t\n':
            start += 1
        while end > 0 and query[end - 1] in ' \t\n':
            end -= 1
        if start >= end:
            raise Diagnostic(DiagnosticTypes.sru, 10)
        iOpPos, strOp = self.find_operator(query, start, end)
        if iOpPos == -1:
            if query[start] == '(' and query[end - 1] == ')':
                return self.translate_fcsql(query, config, basicSearch=basicSearch, start=start + 1, end=end - 1)
            else:
                return self.term_query(query[start:end], config)
        if strOp in ('AND', 'OR'):
            resultLeft = self.translate_fcsql(query, config, basicSearch=basicSearch, start=start, end=iOpPos)
            resultRight = self.translate_fcsql(query, config, basicSearch=basicSearch, start=iOpPos + len(strOp), end=end)
            if len(resultLeft) <= 0 or len(resultRight) <= 0:
                raise Diagnostic(DiagnosticTypes.sru, 10)
            return self.binary_bool(strOp, resultLeft, resultRight, config)
        elif strOp == 'NOT':
            resultRight = self.translate_fcsql(query, config, basicSearch=basicSearch, start=iOpPos + len(strOp),
                                               end=end)
            return self.not_bool(resultRight)
        return {}


if __name__ == '__main__':
    pass

